class TalltyImportExport::ExportPayload::Cell
  attr_accessor :cell_cluster, :value, :header, :parent_path

  # cell_cluster 是表格纵向 集合
  def initialize header, value, context, parent_path=[], &value_handler
    @value = value
    @header = header
    @value_handler = value_handler

    # context
    @context = context

    value.seq_chain.each do |s|
    # s = seq
      @context[:header_seq_to_value_seq_to_value][header.seq] ||= {}
      @context[:header_seq_to_value_seq_to_value][header.seq][s] = value # NOTE: value 对象
    end

    # parent
    @parent_path = parent_path

    @context[:parent_cell_seq_to_children_cell_value_array][parent_cell_seq] ||= []
    @context[:parent_cell_seq_to_children_cell_value_array][parent_cell_seq].push(@value)

    @cell_cluster = []
  end

  def seq
    @seq ||= SecureRandom.uuid
  end

  def parent_cell_seq
    @parent_path.last&.seq
  end

  def depth
    @parent_path.count
  end

  def header_chain_seqs
    [*@parent_path, self].map { |cell| cell.header.cluster_seqs }.reduce(:concat)
  end

  def divise
    if @header.children.present? && @value.value
      new_cells = @header.children.map do |child_header|
        # 分离 header 属性，保留 proc
        header_attrs = child_header.as_json.symbolize_keys
        header_attrs[:proc] = child_header.proc if child_header.proc.present?

        val = @value_handler.call(@value.value, header_attrs, keep_array: true)
        if (child_header.children.present? && val.is_a?(Array))
          val.map do |val|
            cell = TalltyImportExport::ExportPayload::Cell.new(
              child_header,
              TalltyImportExport::ExportPayload::Value.new(val, @value.chain, inherited: true),
              @context,
              [*self.parent_path, self],
              &@value_handler
            )
            cell
          end
        else
          # 这里也需要保留 proc
          header_attrs = child_header.as_json.symbolize_keys
          header_attrs[:proc] = child_header.proc if child_header.proc.present?
          val = @value_handler.call(@value.value, header_attrs)

          TalltyImportExport::ExportPayload::Cell.new(
            child_header,
            TalltyImportExport::ExportPayload::Value.new(val, @value.chain),
            @context,
            [*self.parent_path, self],
            &@value_handler
          )
        end
      end.flatten
      new_cells.each(&:divise)
      @cell_cluster.concat(new_cells.map(&:cell_cluster))
    else
      @cell_cluster = [self]
    end

    self
  end

  def grow_to_line
    @context[:flatten_headers].
      select do |header|
        header_chain_seqs.include?(header.seq)
      end.
      map do |header|
        target_value = nil

        # 兄弟细胞上先查找，以解决同级兄弟细胞父一样，而导致的值覆盖
        brother_cell_value_seqs = (@context[:parent_cell_seq_to_children_cell_value_array][parent_cell_seq] || []).map(&:seq)

        [*brother_cell_value_seqs, *@value.seq_chain.uniq.reverse].each do |seq|
          next if target_value
          # seq = @value.seq
          if @context[:header_seq_to_value_seq_to_value][header.seq]&.[](seq)
            target_value = @context[:header_seq_to_value_seq_to_value][header.seq][seq]
          end
      end

      target_value
    end
  end
end
